@@ -3,7 +3,10 @@
 ## and while Boost Date_Time instantiates the formats relative to a locale
 ## it remains tricky to change TZ on the fly
 
-library(anytime)                        # load it early to test verseion
+tz <- "America/Chicago"
+Sys.setenv("TZ"=tz)
+library(anytime)
+anytime:::setTZ(tz)
 
 ## We turn off tests on Solaris with some regret, yet firmly, as the
 ## combined inability of CRAN to provide us a test platform (to
@@ -11,7 +14,7 @@ library(anytime)                        # load it early to test verseion
 ## tests gives us no choice
 isSolaris <- Sys.info()[["sysname"]] == "SunOS"
 isWindows <- Sys.info()[["sysname"]] == "Windows"
-isRelease <- length(unclass(packageVersion("anytime"))[[1]]) == 3
+isRelease <- length(unclass(utils::packageVersion("anytime"))[[1]]) == 3
 
 if (!isSolaris && !isWindows && !isRelease) {
 
@@ -603,6 +606,7 @@ if (!isSolaris && !isWindows && !isRelease) {
     )
 
     fullPOSIXctEx <- function(tstr) {
+        at <- format(anytime(tstr, tz=tzvec[1]))
         for (tz in tzvec) {
             ## for anytime, the returned POSIXct object has the 'target' timezone (default is local)
             at <- format(   anytime(tstr, tz=tz)        )
